@@ -16,6 +16,6 @@ mysubdirs=`echo $mysubdirs`
 for F in $mysubdirs
 do
 	echo "Configuring in $F..."
-	(cd $F && grep "^AC_CONFIG_HEADER" configure.in > /dev/null && autoheader -I$parentdir)
+	(cd $F && grep "^AC_CONFIG_HEADER" configure.ac > /dev/null || exit 0; autoheader -I$parentdir)
 	(cd $F && autoconf -I$parentdir)
 done
