@@ -44,11 +44,11 @@
 
 #ifdef HAVE_SYS_PTRACE_H
 #  include <sys/ptrace.h>
-#  if !defined(PTRACE_ATTACH) && defined(PT_ATTACH)
-#    define PTRACE_ATTACH PT_ATTACH
+#  if !defined(PT_ATTACH) && defined(PTRACE_ATTACH)
+#    define PT_ATTACH PTRACE_ATTACH
 #  endif
-#  if !defined(PTRACE_DETACH) && defined(PT_DETACH)
-#    define PTRACE_DETACH PT_DETACH
+#  if !defined(PT_DETACH) && defined(PTRACE_DETACH)
+#    define PT_DETACH PTRACE_DETACH
 #  endif
 #endif
 
@@ -196,7 +196,7 @@ static int fr_get_debug_state(void)
 		 *	If we don't do it in that order the read in the parent triggers
 		 *	a SIGKILL.
 		 */
-		if (_PTRACE(PTRACE_ATTACH, ppid) == 0) {
+		if (_PTRACE(PT_ATTACH, ppid) == 0) {
 			/* Wait for the parent to stop */
 			waitpid(ppid, NULL, 0);
 
@@ -206,7 +206,7 @@ static int fr_get_debug_state(void)
 			}
 
 			/* Detach */
-			_PTRACE(PTRACE_DETACH, ppid);
+			_PTRACE(PT_DETACH, ppid);
 			exit(0);
 		}
 
