@@ -1,7 +1,7 @@
 /*
  * misc.c	Various miscellaneous functions.
  *
- * Version:	$Id: af6ee2ce933d56cf6c3695ce10940d29c6f7c230 $
+ * Version:	$Id: 5b88d914557aa3f958693560ddb7c428eddaaf16 $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,7 @@
  * Copyright 2000,2006  The FreeRADIUS server project
  */
 
-RCSID("$Id: af6ee2ce933d56cf6c3695ce10940d29c6f7c230 $")
+RCSID("$Id: 5b88d914557aa3f958693560ddb7c428eddaaf16 $")
 
 #include <freeradius-devel/libradius.h>
 
@@ -79,6 +79,8 @@ typedef struct fr_talloc_link {
  * @param sig to set handler for.
  * @param func handler to set.
  */
+DIAG_OPTIONAL
+DIAG_OFF(disabled-macro-expansion)
 int fr_set_signal(int sig, sig_t func)
 {
 #ifdef HAVE_SIGACTION
@@ -101,6 +103,7 @@ int fr_set_signal(int sig, sig_t func)
 #endif
 	return 0;
 }
+DIAG_ON(disabled-macro-expansion)
 
 /** Uninstall a signal for a specific handler
  *
@@ -108,6 +111,8 @@ int fr_set_signal(int sig, sig_t func)
  *
  * @param sig SIGNAL
  */
+DIAG_OPTIONAL
+DIAG_OFF(disabled-macro-expansion)
 int fr_unset_signal(int sig)
 {
 #ifdef HAVE_SIGACTION
@@ -123,6 +128,7 @@ int fr_unset_signal(int sig)
         return signal(sig, SIG_DFL);
 #endif
 }
+DIAG_ON(disabled-macro-expansion)
 
 static int _fr_trigger_talloc_ctx_free(fr_talloc_link_t *trigger)
 {
@@ -302,10 +308,12 @@ static int ip_prefix_from_str(char const *str, uint32_t *paddr)
 }
 
 
-/** Parse an IPv4 address or IPv4 prefix in presentation format (and others)
+/**
+ * Parse an IPv4 address, IPv4 prefix in presentation format (and others), or
+ * a hostname.
  *
  * @param out Where to write the ip address value.
- * @param value to parse, may be dotted quad [+ prefix], or integer, or octal number, or '*' (INADDR_ANY).
+ * @param value to parse, may be dotted quad [+ prefix], or integer, or octal number, or '*' (INADDR_ANY), or a hostname.
  * @param inlen Length of value, if value is \0 terminated inlen may be -1.
  * @param resolve If true and value doesn't look like an IP address, try and resolve value as a hostname.
  * @param fallback to IPv6 resolution if no A records can be found.
@@ -317,8 +325,8 @@ int fr_pton4(fr_ipaddr_t *out, char const *value, ssize_t inlen, bool resolve, b
 	unsigned int mask;
 	char *eptr;
 
-	/* Dotted quad + / + [0-9]{1,2} */
-	char buffer[INET_ADDRSTRLEN + 3];
+	/* Dotted quad + / + [0-9]{1,2} or a hostname (RFC1035 2.3.4 Size limits) */
+	char buffer[256];
 
 	/*
 	 *	Copy to intermediary buffer if we were given a length
@@ -400,7 +408,9 @@ int fr_pton4(fr_ipaddr_t *out, char const *value, ssize_t inlen, bool resolve, b
 	return 0;
 }
 
-/** Parse an IPv6 address or IPv6 prefix in presentation format (and others)
+/**
+ * Parse an IPv6 address or IPv6 prefix in presentation format (and others),
+ * or a hostname.
  *
  * @param out Where to write the ip address value.
  * @param value to parse.
@@ -415,8 +425,8 @@ int fr_pton6(fr_ipaddr_t *out, char const *value, ssize_t inlen, bool resolve, b
 	unsigned int prefix;
 	char *eptr;
 
-	/* IPv6  + / + [0-9]{1,3} */
-	char buffer[INET6_ADDRSTRLEN + 4];
+	/* IPv6  + / + [0-9]{1,3} or a hostname (RFC1035 2.3.4 Size limits) */
+	char buffer[256];
 
 	/*
 	 *	Copy to intermediary buffer if we were given a length
@@ -622,7 +632,7 @@ do_port:
 	 *	input length indicates there are more than 5 chars
 	 *	after the ':' then there's an issue.
 	 */
-	if (inlen > ((q + sizeof(buffer)) - value)) {
+	if (len > (size_t) ((q + sizeof(buffer)) - value)) {
 	error:
 		fr_strerror_printf("IP string contains trailing garbage after port delimiter");
 		return -1;
@@ -643,7 +653,7 @@ do_port:
 	return 0;
 }
 
-int fr_ntop(char *out, size_t outlen, fr_ipaddr_t *addr)
+int fr_ntop(char *out, size_t outlen, fr_ipaddr_t const *addr)
 {
 	char buffer[INET6_ADDRSTRLEN];
 
@@ -1867,7 +1877,7 @@ static char *mystrtok(char **ptr, char const *sep)
  */
 int fr_get_time(char const *date_str, time_t *date)
 {
-	int		i;
+	int		i, j;
 	time_t		t;
 	struct tm	*tm, s_tm;
 	char		buf[64];
@@ -1924,9 +1934,9 @@ int fr_get_time(char const *date_str, time_t *date)
 			f[0] = f[i];
 			f[i] = p;
 
-			for (i = 0; i < 12; i++) {
-				if (strncasecmp(months[i], f[0], 3) == 0) {
-					tm->tm_mon = i;
+			for (j = 0; j < 12; j++) {
+				if (strncasecmp(months[j], f[0], 3) == 0) {
+					tm->tm_mon = j;
 					break;
 				}
 			}
