@@ -15,7 +15,7 @@
  */
 
 /**
- * $Id: 919cc860eab28fc2e2635dc629f507b7eaf3578c $
+ * $Id: 90c39be69c1f7eea88811ef79cfb020543b88eef $
  *
  * @brief #VALUE_PAIR template functions
  * @file main/tmpl.c
@@ -24,7 +24,7 @@
  *
  * @copyright 2014-2015 The FreeRADIUS server project
  */
-RCSID("$Id: 919cc860eab28fc2e2635dc629f507b7eaf3578c $")
+RCSID("$Id: 90c39be69c1f7eea88811ef79cfb020543b88eef $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/rad_assert.h>
@@ -1676,10 +1676,10 @@ size_t tmpl_prints(char *out, size_t outlen, vp_tmpl_t const *vpt, DICT_ATTR con
 	case TMPL_TYPE_LIST:
 		out[0] = '&';
 		if (vpt->tmpl_request == REQUEST_CURRENT) {
-			snprintf(out + 1, outlen - 1, "%s",
+			snprintf(out + 1, outlen - 1, "%s:",
 				 fr_int2str(pair_lists, vpt->tmpl_list, ""));
 		} else {
-			snprintf(out + 1, outlen - 1, "%s.%s",
+			snprintf(out + 1, outlen - 1, "%s.%s:",
 				 fr_int2str(request_refs, vpt->tmpl_request, ""),
 				 fr_int2str(pair_lists, vpt->tmpl_list, ""));
 		}
