@@ -1,7 +1,7 @@
 /*
  * eap.c    rfc2284 & rfc2869 implementation
  *
- * Version:     $Id: d9660ed4234eb7a338e73562d938e99e3ab31275 $
+ * Version:     $Id: b03654fed8148d9d3933496fa4e99a9f6fc0a1e7 $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -56,7 +56,7 @@
 
 #include <freeradius-devel/modpriv.h>
 
-RCSID("$Id: d9660ed4234eb7a338e73562d938e99e3ab31275 $")
+RCSID("$Id: b03654fed8148d9d3933496fa4e99a9f6fc0a1e7 $")
 
 #include "rlm_eap.h"
 #include <ctype.h>
@@ -125,7 +125,7 @@ int eap_module_instantiate(rlm_eap_t *inst, eap_module_t **m_inst, eap_type_t nu
 	/*
 	 *	Link the loaded EAP-Type
 	 */
-	method->handle = lt_dlopenext(mod_name);
+	method->handle = fr_dlopenext(mod_name);
 	if (!method->handle) {
 		ERROR("rlm_eap (%s): Failed to link %s: %s", inst->xlat_name, mod_name, fr_strerror());
 
