@@ -31,7 +31,7 @@
  * license (including the GNU public license).
  */
 
-RCSID("$Id: 9abae5c80a5204fe099c13995ee0590ce2c1f44f $")
+RCSID("$Id: 3c043f728460d7e1fd8b29fb4211551f6fb4c490 $")
 USES_APPLE_DEPRECATED_API	/* OpenSSL API has been deprecated by Apple */
 
 #include "rlm_eap_pwd.h"
@@ -386,8 +386,8 @@ static int mod_process(void *arg, eap_handler_t *handler)
 		}
 
 		packet = (pwd_id_packet_t *) in;
-		if (in_len < sizeof(packet)) {
-			RDEBUG("Packet is too small (%zd < %zd).", in_len, sizeof(packet));
+		if (in_len < sizeof(*packet)) {
+			RDEBUG("Packet is too small (%zd < %zd).", in_len, sizeof(*packet));
 			return 0;
 		}
 
