@@ -1,7 +1,7 @@
 /*
  * rlm_eap_ttls.c  contains the interfaces that are called from eap
  *
- * Version:     $Id: 912060f0a60bb2951ec5d06f6048e520ddbb6d0b $
+ * Version:     $Id: 524e874e7187f505fbafb81b4ebf16fbbe27e039 $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  *   Copyright 2006 The FreeRADIUS server project
  */
 
-RCSID("$Id: 912060f0a60bb2951ec5d06f6048e520ddbb6d0b $")
+RCSID("$Id: 524e874e7187f505fbafb81b4ebf16fbbe27e039 $")
 
 #include "eap_ttls.h"
 #include "eap_chbind.h"
@@ -1183,7 +1183,7 @@ int eapttls_process(eap_handler_t *handler, tls_session_t *tls_session)
 		if (req->response) {
 			RDEBUG("sending chbind response");
 			fr_pair_add(&fake->reply->vps,
-				eap_chbind_packet2vp(fake, req->response));
+				    eap_chbind_packet2vp(fake->reply, req->response));
 		} else {
 			RDEBUG("no chbind response");
 		}
