@@ -15,7 +15,7 @@
  */
 
 /**
- * $Id: fafcd63533b3b674584c902fce774bae17820f3a $
+ * $Id: 3bf9be2fc393788f0a22b104410af0523d5bba25 $
  * @file rlm_sql.c
  * @brief Implements SQL 'users' file, and SQL accounting.
  *
@@ -24,7 +24,7 @@
  * @copyright 2000  Mike Machado <mike@innercite.com>
  * @copyright 2000  Alan DeKok <aland@ox.org>
  */
-RCSID("$Id: fafcd63533b3b674584c902fce774bae17820f3a $")
+RCSID("$Id: 3bf9be2fc393788f0a22b104410af0523d5bba25 $")
 
 #include <ctype.h>
 
@@ -154,6 +154,7 @@ static ssize_t sql_xlat(void *instance, REQUEST *request, char const *query, cha
 	sql_rcode_t		rcode;
 	ssize_t			ret = 0;
 	size_t			len = 0;
+	char const		*p;
 
 	/*
 	 *	Add SQL-User-Name attribute just in case it is needed
@@ -167,13 +168,18 @@ static ssize_t sql_xlat(void *instance, REQUEST *request, char const *query, cha
 
 	rlm_sql_query_log(inst, request, NULL, query);
 
+	/*
+	 *	Trim whitespace for the prefix check
+	 */
+	for (p = query; is_whitespace(p); p++);
+
 	/*
 	 *	If the query starts with any of the following prefixes,
 	 *	then return the number of rows affected
 	 */
-	if ((strncasecmp(query, "insert", 6) == 0) ||
-	    (strncasecmp(query, "update", 6) == 0) ||
-	    (strncasecmp(query, "delete", 6) == 0)) {
+	if ((strncasecmp(p, "insert", 6) == 0) ||
+	    (strncasecmp(p, "update", 6) == 0) ||
+	    (strncasecmp(p, "delete", 6) == 0)) {
 		int numaffected;
 		char buffer[21]; /* 64bit max is 20 decimal chars + null byte */
 
@@ -189,6 +195,7 @@ static ssize_t sql_xlat(void *instance, REQUEST *request, char const *query, cha
 		numaffected = (inst->module->sql_affected_rows)(handle, inst->config);
 		if (numaffected < 1) {
 			RDEBUG("SQL query affected no rows");
+			(inst->module->sql_finish_query)(handle, inst->config);
 
 			goto finish;
 		}
@@ -225,7 +232,10 @@ static ssize_t sql_xlat(void *instance, REQUEST *request, char const *query, cha
 	if (rcode != RLM_SQL_OK) goto query_error;
 
 	rcode = rlm_sql_fetch_row(inst, request, &handle);
-	if (rcode) goto query_error;
+	if (rcode < 0) {
+		(inst->module->sql_finish_select_query)(handle, inst->config);
+		goto query_error;
+	}
 
 	row = handle->row;
 	if (!row) {
@@ -282,10 +292,18 @@ static int generate_sql_clients(rlm_sql_t *inst)
 
 	if (rlm_sql_select_query(inst, NULL, &handle, inst->config->client_query) != RLM_SQL_OK) return -1;
 
-	while ((rlm_sql_fetch_row(inst, NULL, &handle) == 0) && (row = handle->row)) {
+	while ((rlm_sql_fetch_row(inst, NULL, &handle) == RLM_SQL_OK) && (row = handle->row)) {
+		int num_rows;
 		char *server = NULL;
+
 		i++;
 
+		num_rows = (inst->module->sql_num_fields)(handle, inst->config);
+		if (num_rows < 5) {
+			WARN("SELECT returned too few rows.  Please do not edit 'client_query'");
+			continue;
+		}
+
 		/*
 		 *  The return data for each row MUST be in the following order:
 		 *
@@ -313,7 +331,7 @@ static int generate_sql_clients(rlm_sql_t *inst)
 			continue;
 		}
 
-		if (((inst->module->sql_num_fields)(handle, inst->config) > 5) && (row[5] != NULL) && *row[5]) {
+		if ((num_rows > 5) && (row[5] != NULL) && *row[5]) {
 			server = row[5];
 		}
 
@@ -498,7 +516,7 @@ int sql_set_user(rlm_sql_t *inst, REQUEST *request, char const *username)
 
 	fr_pair_value_strsteal(vp, expanded);
 	RDEBUG2("SQL-User-Name set to '%s'", vp->vp_strvalue);
-	vp->op = T_OP_SET;	
+	vp->op = T_OP_SET;
 
 	/*
 	 *	Delete any existing SQL-User-Name, and replace it with ours.
@@ -535,7 +553,7 @@ static int sql_get_grouplist(rlm_sql_t *inst, rlm_sql_handle_t **handle, REQUEST
 	talloc_free(expanded);
 	if (ret != RLM_SQL_OK) return -1;
 
-	while (rlm_sql_fetch_row(inst, request, handle) == 0) {
+	while (rlm_sql_fetch_row(inst, request, handle) == RLM_SQL_OK) {
 		row = (*handle)->row;
 		if (!row)
 			break;
@@ -568,7 +586,7 @@ static int sql_get_grouplist(rlm_sql_t *inst, rlm_sql_handle_t **handle, REQUEST
 
 /*
  * sql groupcmp function. That way we can do group comparisons (in the users file for example)
- * with the group memberships reciding in sql
+ * with the group memberships residing in sql
  * The group membership query should only return one element which is the username. The returned
  * username will then be checked with the passed check string.
  */
@@ -843,7 +861,7 @@ static int mod_bootstrap(CONF_SECTION *conf, void *instance)
 	 *
 	 *	We need this to check if the sql_fields callback is provided.
 	 */
-	inst->handle = lt_dlopenext(inst->config->sql_driver_name);
+	inst->handle = fr_dlopenext(inst->config->sql_driver_name);
 	if (!inst->handle) {
 		ERROR("Could not link driver %s: %s", inst->config->sql_driver_name, fr_strerror());
 		ERROR("Make sure it (and all its dependent libraries!) are in the search path of your system's ld");
@@ -901,6 +919,52 @@ static int mod_bootstrap(CONF_SECTION *conf, void *instance)
 }
 
 
+static void *mod_conn_create(TALLOC_CTX *ctx, void *instance)
+{
+	int rcode;
+	rlm_sql_t *inst = instance;
+	rlm_sql_handle_t *handle;
+
+	/*
+	 *	Connections cannot be alloced from the inst or
+	 *	pool contexts due to threading issues.
+	 */
+	handle = talloc_zero(ctx, rlm_sql_handle_t);
+	if (!handle) return NULL;
+
+	handle->log_ctx = talloc_pool(handle, 2048);
+	if (!handle->log_ctx) {
+		talloc_free(handle);
+		return NULL;
+	}
+
+	/*
+	 *	Handle requires a pointer to the SQL inst so the
+	 *	destructor has access to the module configuration.
+	 */
+	handle->inst = inst;
+
+	rcode = (inst->module->sql_socket_init)(handle, inst->config);
+	if (rcode != 0) {
+	fail:
+		exec_trigger(NULL, inst->cs, "modules.sql.fail", true);
+
+		/*
+		 *	Destroy any half opened connections.
+		 */
+		talloc_free(handle);
+		return NULL;
+	}
+
+	if (inst->config->connect_query) {
+		if (rlm_sql_select_query(inst, NULL, &handle, inst->config->connect_query) != RLM_SQL_OK) goto fail;
+		(inst->module->sql_finish_select_query)(handle, inst->config);
+	}
+
+	return handle;
+}
+
+
 static int mod_instantiate(CONF_SECTION *conf, void *instance)
 {
 	rlm_sql_t *inst = instance;
@@ -1170,7 +1234,7 @@ static rlm_rcode_t mod_authorize(void *instance, REQUEST *request)
 	}
 
 	/*
-	 *	Neither group checks or profiles will work without
+	 *	Neither group checks nor profiles will work without
 	 *	a group membership query.
 	 */
 	if (!inst->config->groupmemb_query) goto release;
@@ -1263,7 +1327,7 @@ skipreply:
 	}
 
 	/*
-	 *	At this point the key (user) hasn't be found in the check table, the reply table
+	 *	At this point the key (user) hasn't been found in the check table, the reply table
 	 *	or the group mapping table, and there was no matching profile.
 	 */
 release:
@@ -1399,7 +1463,7 @@ static int acct_redundant(rlm_sql_t *inst, REQUEST *request, sql_acct_section_t
 		case RLM_SQL_ERROR:
 		/*
 		 *  If we get RLM_SQL_RECONNECT it means all connections in the pool
-		 *  were exhausted, and we couldn't create a new connection,
+		 *  were exhausted and we couldn't create a new connection,
 		 *  so we do not need to call fr_connection_release.
 		 */
 		case RLM_SQL_RECONNECT:
@@ -1431,7 +1495,7 @@ static int acct_redundant(rlm_sql_t *inst, REQUEST *request, sql_acct_section_t
 		(inst->module->sql_finish_query)(handle, inst->config);
 		RDEBUG("%i record(s) updated", numaffected);
 
-		if (numaffected > 0) break;	/* A query succeeded, were done! */
+		if (numaffected > 0) break;	/* A query succeeded, we're done! */
 	next:
 		/*
 		 *  We assume all entries with the same name form a redundant
@@ -1590,12 +1654,22 @@ static rlm_rcode_t mod_checksimul(void *instance, REQUEST * request)
 		call_num = vp->vp_strvalue;
 	}
 
-	while (rlm_sql_fetch_row(inst, request, &handle) == 0) {
+	while (rlm_sql_fetch_row(inst, request, &handle) == RLM_SQL_OK) {
+		int num_rows;
+
 		row = handle->row;
 		if (!row) {
 			break;
 		}
 
+		num_rows = (inst->module->sql_num_fields)(handle, inst->config);
+		if (num_rows < 8) {
+			RDEBUG("Too few rows returned.  Please do not edit 'simul_verify_query'");
+			rcode = RLM_MODULE_FAIL;
+
+			goto finish;
+		}
+
 		if (!row[2]){
 			RDEBUG("Cannot zap stale entry. No username present in entry");
 			rcode = RLM_MODULE_FAIL;
@@ -1636,7 +1710,7 @@ static rlm_rcode_t mod_checksimul(void *instance, REQUEST * request)
 					else if (strcmp(row[7], "SLIP") == 0)
 						proto = 'S';
 				}
-				if (row[8])
+				if ((num_rows > 8) && row[8])
 					sess_time = atoi(row[8]);
 				session_zap(request, nas_addr, nas_port,
 					    row[2], row[1], framed_addr,
